@@ -3,11 +3,9 @@
  * Copyright 2006 Rob Landley <rob@landley.net>
  */
 
-// NOTE: If option parsing segfaults, switch on TOYBOX_DEBUG in menuconfig.
-
-// Enabling TOYBOX_DEBUG in .config adds syntax checks to option string parsing
-// which aren't needed in the final code (your option string is hardwired and
-// should be correct when you ship), but are useful for development.
+// NOTE: If option parsing segfaults, switch on TOYBOX_DEBUG in menuconfig to
+// add syntax checks to option string parsing which aren't needed in the final
+// code (since get_opt string is hardwired and should be correct when you ship)
 
 #include "toys.h"
 
@@ -67,23 +65,28 @@
 //     % time offset in milliseconds with optional s/m/h/d suffix
 //     (longopt)
 //     | this is required. If more than one marked, only one required.
-//     ; long option's argument is optional (can only be supplied with --opt=)
+//     ; Option's argument is optional, and must be collated: -aARG or --a=ARG
 //     ^ Stop parsing after encountering this argument
 //    " " (space char) the "plus an argument" must be separate
 //        I.E. "-j 3" not "-j3". So "kill -stop" != "kill -s top"
 //
 //   At the beginning of the get_opt string (before any options):
-//     ^ stop at first nonoption argument
 //     <0 die if less than # leftover arguments (default 0)
 //     >9 die if > # leftover arguments (default MAX_INT)
-//     ? Allow unknown arguments (pass them through to command).
+//     0 Include argv[0] in optargs
+//     ^ stop at first nonoption argument
+//     ? Pass unknown arguments through to command (implied when no flags).
 //     & first arg has imaginary dash (ala tar/ps/ar) which sets FLAGS_NODASH
+//     ~ Collate following bare longopts (as if under short opt, repeatable)
 //
 //   At the end: [groups] of previously seen options
 //     - Only one in group (switch off)    [-abc] means -ab=-b, -ba=-a, -abc=-c
 //     + Synonyms (switch on all)          [+abc] means -ab=-abc, -c=-abc
 //     ! More than one in group is error   [!abc] means -ab calls error_exit()
 //       primarily useful if you can switch things back off again.
+//
+//   You may use octal escapes with the high bit (128) set to use a control
+//   character as an option flag. For example, \300 would be the option -@
 
 // Notes from getopt man page
 //   - and -- cannot be arguments.
@@ -128,25 +131,31 @@ struct getoptflagstate
   unsigned excludes, requires;
 };
 
-// Use getoptflagstate to parse parse one command line option from argv
-static int gotflag(struct getoptflagstate *gof, struct opts *opt)
+static void forget_arg(struct opts *opt)
 {
+  if (opt->arg) {
+    if (opt->type=='*') llist_traverse((void *)*opt->arg, free);
+    *opt->arg = opt->val[2].l;
+  }
+}
+
+// Use getoptflagstate to parse one command line option from argv
+// Sets flags, saves/clears opt->arg, advances gof->arg/gof->argc as necessary
+static void gotflag(struct getoptflagstate *gof, struct opts *opt, int longopt)
+{
+  unsigned long long i;
+  struct opts *and;
+  char *arg;
   int type;
 
   // Did we recognize this option?
-  if (!opt) {
-    if (gof->noerror) return 1;
-    help_exit("Unknown option %s", gof->arg);
-  }
+  if (!opt) help_exit("Unknown option '%s'", gof->arg);
 
   // Might enabling this switch off something else?
   if (toys.optflags & opt->dex[0]) {
-    struct opts *clr;
-    unsigned long long i = 1;
-
     // Forget saved argument for flag we switch back off
-    for (clr=gof->opts, i=1; clr; clr = clr->next, i<<=1)
-      if (clr->arg && (i & toys.optflags & opt->dex[0])) *clr->arg = 0;
+    for (and = gof->opts, i = 1; and; and = and->next, i<<=1)
+      if (i & toys.optflags & opt->dex[0]) forget_arg(and);
     toys.optflags &= ~opt->dex[0];
   }
 
@@ -156,87 +165,80 @@ static int gotflag(struct getoptflagstate *gof, struct opts *opt)
   if (opt->flags&2) gof->stopearly=2;
 
   if (toys.optflags & gof->excludes) {
-    struct opts *bad;
-    unsigned i = 1;
-
-    for (bad=gof->opts, i=1; bad ;bad = bad->next, i<<=1) {
-      if (opt == bad || !(i & toys.optflags)) continue;
-      if (toys.optflags & bad->dex[2]) break;
+    for (and = gof->opts, i = 1; and; and = and->next, i<<=1) {
+      if (opt == and || !(i & toys.optflags)) continue;
+      if (toys.optflags & and->dex[2]) break;
     }
-    if (bad) help_exit("No '%c' with '%c'", opt->c, bad->c);
+    if (and) help_exit("No '%c' with '%c'", opt->c, and->c);
   }
 
-  // Does this option take an argument?
-  if (!gof->arg) {
-    if (opt->flags & 8) return 0;
-    gof->arg = "";
-  } else gof->arg++;
-  type = opt->type;
+  // Are we NOT saving an argument? (Type 0, '@', unattached ';', short ' ')
+  if (*(arg = gof->arg)) gof->arg++;
+  if ((type = opt->type) == '@') {
+    ++*opt->arg;
+    return;
+  }
+  if (!longopt && *gof->arg && (opt->flags & 4)) return forget_arg(opt);
+  if (!type || (!arg[!longopt] && (opt->flags & 8))) return forget_arg(opt);
 
-  if (type == '@') ++*(opt->arg);
-  else if (type) {
-    char *arg = gof->arg;
+  // Handle "-xblah" and "-x blah", but also a third case: "abxc blah"
+  // to make "tar xCjfv blah1 blah2 thingy" work like
+  // "tar -x -C blah1 -j -f blah2 -v thingy"
 
-    // Handle "-xblah" and "-x blah", but also a third case: "abxc blah"
-    // to make "tar xCjfv blah1 blah2 thingy" work like
-    // "tar -x -C blah1 -j -f blah2 -v thingy"
+  if (longopt && *arg) arg++;
+  else arg = (gof->nodash_now||!*gof->arg) ? toys.argv[++gof->argc] : gof->arg;
+  if (!gof->nodash_now) gof->arg = "";
+  if (!arg) {
+    struct longopts *lo;
 
-    if (gof->nodash_now || (!arg[0] && !(opt->flags & 8)))
-      arg = toys.argv[++gof->argc];
-    if (!arg) {
-      char *s = "Missing argument to ";
-      struct longopts *lo;
-
-      if (opt->c != -1) help_exit("%s-%c", s, opt->c);
-
-      for (lo = gof->longopts; lo->opt != opt; lo = lo->next);
-      help_exit("%s--%.*s", s, lo->len, lo->str);
-    }
-
-    if (type == ':') *(opt->arg) = (long)arg;
-    else if (type == '*') {
-      struct arg_list **list;
-
-      list = (struct arg_list **)opt->arg;
-      while (*list) list=&((*list)->next);
-      *list = xzalloc(sizeof(struct arg_list));
-      (*list)->arg = arg;
-    } else if (type == '#' || type == '-') {
-      long l = atolx(arg);
-      if (type == '-' && !ispunct(*arg)) l*=-1;
-      if (l < opt->val[0].l) help_exit("-%c < %ld", opt->c, opt->val[0].l);
-      if (l > opt->val[1].l) help_exit("-%c > %ld", opt->c, opt->val[1].l);
-
-      *(opt->arg) = l;
-    } else if (CFG_TOYBOX_FLOAT && type == '.') {
-      FLOAT *f = (FLOAT *)(opt->arg);
-
-      *f = strtod(arg, &arg);
-      if (opt->val[0].l != LONG_MIN && *f < opt->val[0].f)
-        help_exit("-%c < %lf", opt->c, (double)opt->val[0].f);
-      if (opt->val[1].l != LONG_MAX && *f > opt->val[1].f)
-        help_exit("-%c > %lf", opt->c, (double)opt->val[1].f);
-    } else if (type=='%') *(opt->arg) = xparsemillitime(arg);
-
-    if (!gof->nodash_now) gof->arg = "";
+    arg = "Missing argument to ";
+    if (opt->c != -1) help_exit("%s-%c", arg, opt->c);
+    for (lo = gof->longopts; lo->opt != opt; lo = lo->next);
+    help_exit("%s--%.*s", arg, lo->len, lo->str);
   }
 
-  return 0;
+  // Parse argument by type
+  if (type == ':') *(opt->arg) = (long)arg;
+  else if (type == '*') {
+    struct arg_list **list;
+
+    list = (struct arg_list **)opt->arg;
+    while (*list) list=&((*list)->next);
+    *list = xzalloc(sizeof(struct arg_list));
+    (*list)->arg = arg;
+  } else if (type == '#' || type == '-' || type == '%') {
+    long long l = (type == '%') ? xparsemillitime(arg) : atolx(arg);
+
+    if (type == '-' && !ispunct(*arg)) l*=-1;
+    arg = (type == '%') ? "ms" : "";
+    if (l < opt->val[0].l) help_exit("-%c < %ld%s", opt->c, opt->val[0].l, arg);
+    if (l > opt->val[1].l) help_exit("-%c > %ld%s", opt->c, opt->val[1].l, arg);
+
+    *(opt->arg) = l;
+  } else if (CFG_TOYBOX_FLOAT && type == '.') {
+    FLOAT *f = (FLOAT *)(opt->arg);
+
+    *f = strtod(arg, &arg);
+    if (opt->val[0].l != LONG_MIN && *f < opt->val[0].f)
+      help_exit("-%c < %lf", opt->c, (double)opt->val[0].f);
+    if (opt->val[1].l != LONG_MAX && *f > opt->val[1].f)
+      help_exit("-%c > %lf", opt->c, (double)opt->val[1].f);
+  }
 }
 
 // Parse this command's options string into struct getoptflagstate, which
 // includes a struct opts linked list in reverse order (I.E. right-to-left)
-void parse_optflaglist(struct getoptflagstate *gof)
+static int parse_optflaglist(struct getoptflagstate *gof)
 {
   char *options = toys.which->options;
   long *nextarg = (long *)&this;
   struct opts *new = 0;
-  int idx;
+  int idx, rc = 0;
 
   // Parse option format string
   memset(gof, 0, sizeof(struct getoptflagstate));
   gof->maxargs = INT_MAX;
-  if (!options) return;
+  if (!options) return 0;
 
   // Parse leading special behavior indicators
   for (;;) {
@@ -244,14 +246,15 @@ void parse_optflaglist(struct getoptflagstate *gof)
     else if (*options == '<') gof->minargs=*(++options)-'0';
     else if (*options == '>') gof->maxargs=*(++options)-'0';
     else if (*options == '?') gof->noerror++;
-    else if (*options == '&') toys.optflags |= FLAGS_NODASH;
+    else if (*options == '&') gof->nodash_now = 1;
+    else if (*options == '0') rc = 1;
     else break;
     options++;
   }
 
   // Parse option string into a linked list of options with attributes.
 
-  if (!*options) gof->stopearly++;
+  if (!*options) gof->noerror++;
   while (*options) {
     char *temp;
 
@@ -266,7 +269,7 @@ void parse_optflaglist(struct getoptflagstate *gof)
       new->val[0].l = LONG_MIN;
       new->val[1].l = LONG_MAX;
     }
-    // Each option must start with "(" or an option character.  (Bare
+    // Each option must start with "(" or an option character. (Bare
     // longopts only come at the start of the string.)
     if (*options == '(' && new->c != -1) {
       char *end;
@@ -308,8 +311,8 @@ void parse_optflaglist(struct getoptflagstate *gof)
       } else error_exit("<>= only after .#%%");
       options = --temp;
 
-    // At this point, we've hit the end of the previous option.  The
-    // current character is the start of a new option.  If we've already
+    // At this point, we've hit the end of the previous option. The
+    // current character is the start of a new option. If we've already
     // assigned an option to this struct, loop to allocate a new one.
     // (It'll get back here afterwards and fall through to next else.)
     } else if (new->c) {
@@ -326,9 +329,10 @@ void parse_optflaglist(struct getoptflagstate *gof)
   // (This goes right to left so we need the whole list before we can start.)
   idx = 0;
   for (new = gof->opts; new; new = new->next) {
-    unsigned long long u = 1L<<idx++;
+    unsigned long long u = 1LL<<idx++;
 
-    if (new->c == 1) new->c = 0;
+    if (new->c == 1 || new->c=='~') new->c = 0;
+    else new->c &= 127;
     new->dex[1] = u;
     if (new->flags & 1) gof->requires |= u;
     if (new->type) {
@@ -339,7 +343,7 @@ void parse_optflaglist(struct getoptflagstate *gof)
 
   // Parse trailing group indicators
   while (*options) {
-    unsigned bits = 0;
+    unsigned long long bits = 0;
 
     if (CFG_TOYBOX_DEBUG && *options != '[') error_exit("trailing %s", options);
 
@@ -351,26 +355,28 @@ void parse_optflaglist(struct getoptflagstate *gof)
     // Don't advance past ] but do process it once in loop.
     while (*options++ != ']') {
       struct opts *opt;
-      int i;
+      long long ll;
 
       if (CFG_TOYBOX_DEBUG && !*options) error_exit("[ without ]");
       // Find this option flag (in previously parsed struct opt)
-      for (i=0, opt = gof->opts; ; i++, opt = opt->next) {
+      for (ll = 1, opt = gof->opts; ; ll <<= 1, opt = opt->next) {
         if (*options == ']') {
           if (!opt) break;
-          if (bits&(1<<i)) opt->dex[idx] |= bits&~(1<<i);
+          if (bits&ll) opt->dex[idx] |= bits&~ll;
         } else {
           if (*options==1) break;
           if (CFG_TOYBOX_DEBUG && !opt)
             error_exit("[] unknown target %c", *options);
-          if (opt->c == *options) {
-            bits |= 1<<i;
+          if (opt->c == (127&*options)) {
+            bits |= ll;
             break;
           }
         }
       }
     }
   }
+
+  return rc;
 }
 
 // Fill out toys.optflags, toys.optargs, and this[] from toys.argv
@@ -380,27 +386,30 @@ void get_optflags(void)
   struct getoptflagstate gof;
   struct opts *catch;
   unsigned long long saveflags;
-  char *letters[]={"s",""};
+  char *letters[]={"s",""}, *ss;
 
   // Option parsing is a two stage process: parse the option string into
   // a struct opts list, then use that list to process argv[];
 
+  toys.exitval = toys.which->flags >> 24;
+
   // Allocate memory for optargs
-  saveflags = 0;
+  saveflags = toys.optc = parse_optflaglist(&gof);
   while (toys.argv[saveflags++]);
   toys.optargs = xzalloc(sizeof(char *)*saveflags);
+  if (toys.optc) *toys.optargs = *toys.argv;
 
-  parse_optflaglist(&gof);
+  if (toys.argv[1] && toys.argv[1][0] == '-') gof.nodash_now = 0;
 
   // Iterate through command line arguments, skipping argv[0]
   for (gof.argc=1; toys.argv[gof.argc]; gof.argc++) {
     gof.arg = toys.argv[gof.argc];
-    catch = NULL;
+    catch = 0;
 
     // Parse this argument
     if (gof.stopearly>1) goto notflag;
 
-    gof.nodash_now = 0;
+    if (gof.argc>1 || *gof.arg=='-') gof.nodash_now = 0;
 
     // Various things with dashes
     if (*gof.arg == '-') {
@@ -410,6 +419,8 @@ void get_optflags(void)
       gof.arg++;
       if (*gof.arg=='-') {
         struct longopts *lo;
+        struct arg_list *al = 0, *al2;
+        int ii;
 
         gof.arg++;
         // Handle --
@@ -418,52 +429,78 @@ void get_optflags(void)
           continue;
         }
 
-        // do we match a known --longopt?
+        // unambiguously match the start of a known --longopt?
+        check_help(toys.argv+gof.argc);
         for (lo = gof.longopts; lo; lo = lo->next) {
-          if (!strncmp(gof.arg, lo->str, lo->len)) {
-            if (!gof.arg[lo->len]) gof.arg = 0;
-            else if (gof.arg[lo->len] == '=' && lo->opt->type)
-              gof.arg += lo->len;
-            else continue;
-            // It's a match.
-            catch = lo->opt;
-            break;
+          for (ii = 0; ii<lo->len; ii++) if (gof.arg[ii] != lo->str[ii]) break;
+
+          // = only terminates when we can take an argument, not type 0 or '@'
+          if (!gof.arg[ii] || (gof.arg[ii]=='=' && !strchr("@", lo->opt->type)))
+          {
+            al2 = xmalloc(sizeof(struct arg_list));
+            al2->next = al;
+            al2->arg = (void *)lo;
+            al = al2;
+
+            // Exact match is unambigous even when longer options available
+            if (ii==lo->len) {
+              llist_traverse(al, free);
+              al = 0;
+
+              break;
+            }
           }
         }
+        // How many matches?
+        if (al) {
+          *libbuf = 0;
+          if (al->next) for (ss = libbuf, al2 = al; al2; al2 = al2->next) {
+            lo = (void *)al2->arg;
+            ss += sprintf(ss, " %.*s"+(al2==al), lo->len, lo->str);
+          } else lo = (void *)al->arg;
+          llist_traverse(al, free);
+          if (*libbuf) error_exit("bad --%s (%s)", gof.arg, libbuf);
+        }
 
+        // One unambiguous match?
+        if (lo) {
+          catch = lo->opt;
+          while (!strchr("=", *gof.arg)) gof.arg++;
         // Should we handle this --longopt as a non-option argument?
-        if (!lo && gof.noerror) {
+        } else if (gof.noerror) {
           gof.arg -= 2;
           goto notflag;
         }
 
         // Long option parsed, handle option.
-        gotflag(&gof, catch);
+        gotflag(&gof, catch, 1);
         continue;
       }
 
     // Handle things that don't start with a dash.
     } else {
-      if ((toys.optflags & FLAGS_NODASH) && gof.argc == 1) gof.nodash_now = 1;
+      if (gof.nodash_now) toys.optflags |= FLAGS_NODASH;
       else goto notflag;
     }
 
-    // At this point, we have the args part of -args.  Loop through
+    // At this point, we have the args part of -args. Loop through
     // each entry (could be -abc meaning -a -b -c)
     saveflags = toys.optflags;
-    while (*gof.arg) {
+    while (gof.arg && *gof.arg) {
 
       // Identify next option char.
       for (catch = gof.opts; catch; catch = catch->next)
         if (*gof.arg == catch->c)
-          if (!((catch->flags&4) && gof.arg[1])) break;
+          if (!gof.arg[1] || (catch->flags&(4|8))!=4) break;
 
-      // Handle option char (advancing past what was used)
-      if (gotflag(&gof, catch) ) {
+      if (!catch && gof.noerror) {
         toys.optflags = saveflags;
         gof.arg = toys.argv[gof.argc];
         goto notflag;
       }
+
+      // Handle option char (advancing past what was used)
+      gotflag(&gof, catch, 0);
     }
     continue;
 
@@ -490,6 +527,8 @@ notflag:
     help_exit("Needs %s-%s", s[1] ? "one of " : "", needs);
   }
 
+  toys.exitval = 0;
+
   if (CFG_TOYBOX_FREE) {
     llist_traverse(gof.opts, free);
     llist_traverse(gof.longopts, free);
