@@ -213,9 +213,6 @@ int highest_bit(unsigned long l);
 long long peek_le(void *ptr, unsigned size);
 long long peek_be(void *ptr, unsigned size);
 long long peek(void *ptr, unsigned size);
-void poke_le(void *ptr, long long val, unsigned size);
-void poke_be(void *ptr, long long val, unsigned size);
-void poke(void *ptr, long long val, unsigned size);
 struct string_list *find_in_path(char *path, char *filename);
 long long estrtol(char *str, char **end, int base);
 long long xstrtol(char *str, char **end, int base);
