@@ -54,7 +54,8 @@
 // will transitively include it, and ones that don't (macOS) won't break.
 #include <sys/types.h>
 
-// Various constants old build environments might not have even if kernel does
+// Various constants old build environments (or glibc, hiding behind
+// _GNU_SOURCE) might not have even if kernel does.
 
 #ifndef AT_FDCWD             // Kernel commit 5590ff0d5528 2006
 #define AT_FDCWD -100
@@ -68,6 +69,10 @@
 #define AT_REMOVEDIR 0x200
 #endif
 
+#ifndef O_DIRECT
+#define O_DIRECT 0x4000
+#endif
+
 #ifndef RLIMIT_RTTIME // Commit 78f2c7db6068f 2008
 #define RLIMIT_RTTIME 15
 #endif
@@ -305,12 +310,13 @@ typedef enum android_LogPriority {
 #endif
 #if !defined(__BIONIC__) || defined(__ANDROID_NDK__)
 // Android NDKv18 has liblog.so but not liblog.a for static builds.
-static inline int stub_out_log_write(int pri, const char *tag, const char *msg)
+static inline int stub_out_log_buf_write(int log_id, int pri, const char *tag,
+  const char *msg)
 {
   return -1;
 }
 #ifdef __ANDROID_NDK__
-#define __android_log_write(a, b, c) stub_out_log_write(a, b, c)
+#define __android_log_buf_write(a, b, c, d) stub_out_log_buf_write(a, b, c, d)
 #endif
 
 #endif
