@@ -775,7 +775,7 @@ struct group *xgetgrnam(char *name)
 
 void xsetuser(struct passwd *pwd)
 {
-  if (initgroups(pwd->pw_name, pwd->pw_gid) || setgid(pwd->pw_uid)
+  if (initgroups(pwd->pw_name, pwd->pw_gid) || setgid(pwd->pw_gid)
       || setuid(pwd->pw_uid)) perror_exit("xsetuser '%s'", pwd->pw_name);
 }
 
@@ -867,9 +867,9 @@ long long xsendfile_len(int in, int out, long long bytes)
 {
   long long len = sendfile_len(in, out, bytes, 0);
 
-  if (bytes != -1 && bytes != len) {
+  if (len == -1 || (bytes != -1 && bytes != len)) {
     if (out == 1 && len<0) xexit();
-    error_exit("short %s", (len<0) ? "write" : "read");
+    perror_exit("short %s", (len<0) ? "write" : "read");
   }
 
   return len;
