@@ -89,7 +89,7 @@
 #define HELP_HTML    8  // Output HTML
 
 struct toy_list *toy_find(char *name);
-void show_help(FILE *out, int full);
+void show_help(int full);
 void check_help(char **arg);
 void toy_singleinit(struct toy_list *which, char *argv[]);
 void toy_init(struct toy_list *which, char *argv[]);
@@ -141,5 +141,5 @@ extern char **environ, *toybox_version, toybuf[4096], libbuf[4096];
 #ifndef TOYBOX_VENDOR
 #define TOYBOX_VENDOR ""
 #endif
-#define TOYBOX_VERSION "0.8.12"TOYBOX_VENDOR
+#define TOYBOX_VERSION "0.8.13"TOYBOX_VENDOR
 #endif
