@@ -18,25 +18,32 @@ config DEMO_UTF8TOWC
 void demo_utf8towc_main(void)
 {
   mbstate_t mb;
-  int len1, len2;
-  unsigned u, h;
-  wchar_t wc1, wc2;
-
-  setlocale(LC_ALL, "en_US.UTF-8");
+  int len1, len2, maxlen = 0;
+  unsigned h, u, wc2;
+  wchar_t wc1;
+  char *str = (void *)&h;
 
   memset(&mb, 0, sizeof(mb));
-  for (u=1; u; u++) {
-    char *str = (void *)&h;
-
+  // Although there are 0x10ffff unicode points, test all 4 byte combinations.
+  for (u = 1; u;) {
     wc1 = wc2 = 0;
     len2 = 4;
-    h = htonl(u);
-    while (!*str) str++, len2--;
 
+    h = SWAP_BE32(u);
     len1 = mbrtowc(&wc1, str, len2, &mb);
     if (len1<0) memset(&mb, 0, sizeof(mb));
     len2 = utf8towc(&wc2, str, len2);
+
+    if (wcwidth(wc2)>maxlen) maxlen = wcwidth(wc2);
     if (len1 != len2 || wc1 != wc2)
-      printf("%x %d %x %d %x\n", u, len1, wc1, len2, wc2);
+      printf("%x %d %x %d %x\n", u++, len1, wc1, len2, wc2);
+    else if (len2<1) u++;
+    else {
+      h = 1<<(8*(4-len2));
+      u &= ~(h-1);
+      u += h;
+    }
+
   }
+  dprintf(2, "maxlen=%d\n", maxlen);
 }
