@@ -3,6 +3,9 @@
  * Copyright 2015 Rob Landley <rob@landley.net>
  *
  * No standard.
+ *
+ * In 1979 3BSD's tset had a sleep(1) to let mechanical printer-and-ink
+ * terminals "settle down". We're not doing that.
 
 USE_RESET(NEWTOY(reset, 0, TOYFLAG_USR|TOYFLAG_BIN))
 
@@ -12,7 +15,7 @@ config RESET
   help
     usage: reset
 
-    reset the terminal
+    Reset the terminal.
 */
 #include "toys.h"
 
@@ -21,5 +24,8 @@ void reset_main(void)
   int fd = tty_fd();
 
   // man 4 console_codes: reset terminal is ESC (no left bracket) c
-  xwrite(fd<0 ? 1 : fd, "\033c", 2);
+  // DEC private mode set enable wraparound sequence.
+  if (fd<0) fd = 1;
+  xwrite(fd, "\ec\e[?7h", 2);
+  set_terminal(fd, 0, 0, 0);
 }
