@@ -5,13 +5,13 @@
  *
  * See RFC 2132, and 3442 (option 121)
 
-USE_DHCP(NEWTOY(dhcp, "V:H:F:x*r:O*A#<0=20T#<0=3t#<0=3s:p:i:SBRCaovqnbf", TOYFLAG_SBIN|TOYFLAG_ROOTONLY))
+USE_DHCP(NEWTOY(dhcp, "V:H:F:x*r:O*A#<0=20T#<0=3t#<0=3s:p:i:SBRCuaovqnbf", TOYFLAG_SBIN|TOYFLAG_ROOTONLY))
 
 config DHCP
   bool "dhcp"
   default n
   help
-   usage: dhcp [-fbnqvoCRB] [-i IFACE] [-r IP] [-s PROG] [-p PIDFILE]
+   usage: dhcp [-fbnqvouCRB] [-i IFACE] [-r IP] [-s PROG] [-p PIDFILE]
                [-H HOSTNAME] [-V VENDOR] [-x OPT:VAL] [-O OPT]
 
         Configure network dynamically using DHCP.
@@ -38,6 +38,7 @@ config DHCP
       -H Send NAME as client hostname (default none)
       -V VENDOR Vendor identifier (default 'toybox VERSION')
       -C Don't send MAC as client identifier
+      -u Unset broadcast bit (receive unicast offer and ack messages)
       -v Verbose
 
       Signals:
@@ -80,6 +81,7 @@ GLOBALS(
 #define STATE_RENEW_REQUESTED 5
 #define STATE_RELEASED        6
 
+#define BOOTP_UNICAST     0x0
 #define BOOTP_BROADCAST   0x8000
 #define DHCP_MAGIC        0x63825363
 
@@ -957,17 +959,6 @@ static uint32_t get_option_serverid (uint8_t *opt, dhcpc_result_t *presult)
   return var;
 }
 
-static uint8_t get_option_msgtype(uint8_t *opt)
-{
-  uint32_t var = 0;
-  while (*opt != DHCP_OPTION_MSG_TYPE) {
-    if (*opt == DHCP_OPTION_END) return var;
-    opt += opt[1] + 2;
-  }
-  memcpy(&var, opt+2, sizeof(uint8_t));
-  return var;
-}
-
 static uint8_t get_option_lease(uint8_t *opt, dhcpc_result_t *presult)
 {
   uint32_t var = 0;
@@ -988,6 +979,7 @@ static int dhcpc_sendmsg(int msgtype)
   uint8_t *pend;
   struct in_addr rqsd;
   char *vendor;
+  uint16_t broadcast_bit = (toys.optflags & FLAG_u) ? BOOTP_UNICAST : BOOTP_BROADCAST;
 
   // Create the common message header settings
   memset(&state->pdhcp, 0, sizeof(dhcp_msg_t));
@@ -1007,7 +999,7 @@ static int dhcpc_sendmsg(int msgtype)
   // Handle the message specific settings
   switch (msgtype) {
   case DHCPDISCOVER: // Broadcast DISCOVER message to all servers
-    state->pdhcp.flags = htons(BOOTP_BROADCAST); //  Broadcast bit.
+    state->pdhcp.flags = htons(broadcast_bit);
     if (toys.optflags & FLAG_r) {
       inet_aton(TT.req_ip, &rqsd);
       pend = dhcpc_addreqipaddr(&rqsd, pend);
@@ -1022,7 +1014,7 @@ static int dhcpc_sendmsg(int msgtype)
     if (toys.optflags & FLAG_x) pend = set_xopt(pend);
     break;
   case DHCPREQUEST: // Send REQUEST message to the server that sent the *first* OFFER
-    state->pdhcp.flags = htons(BOOTP_BROADCAST); //  Broadcast bit.
+    state->pdhcp.flags = htons(broadcast_bit);
     if (state->status == STATE_RENEWING) memcpy(&state->pdhcp.ciaddr, &state->ipaddr.s_addr, 4);
     pend = dhcpc_addmaxsize(pend, htons(sizeof(dhcp_raw_t)));
     rqsd.s_addr = htonl(server);
@@ -1180,12 +1172,18 @@ static uint8_t dhcpc_parseoptions(dhcpc_result_t *presult, uint8_t *optptr)
 // parses recvd messege to check that it was for us.
 static uint8_t dhcpc_parsemsg(dhcpc_result_t *presult)
 {
+  char *opt = state->pdhcp.options;
+
   if (state->pdhcp.op == DHCP_REPLY
       && !memcmp(state->pdhcp.chaddr, state->macaddr, 6)
       && !memcmp(&state->pdhcp.xid, &xid, sizeof(xid))) {
     memcpy(&presult->ipaddr.s_addr, &state->pdhcp.yiaddr, 4);
     presult->ipaddr.s_addr = ntohl(presult->ipaddr.s_addr);
-    return get_option_msgtype(state->pdhcp.options);
+
+    while (*opt != DHCP_OPTION_END) {
+      if (*opt == DHCP_OPTION_MSG_TYPE) return opt[2];
+      opt += opt[1]+2;
+    }
   }
   return 0;
 }
