@@ -7,6 +7,7 @@
  * Our "unspecified" behavior for no paths is to use "."
  * Parentheses can only stack 4096 deep
  * Not treating two {} as an error, but only using last
+ * TODO: -context
 
 USE_FIND(NEWTOY(find, "?^HL[-HL]", TOYFLAG_USR|TOYFLAG_BIN))
 
@@ -17,38 +18,51 @@ config FIND
     usage: find [-HL] [DIR...] [<options>]
 
     Search directories for matching files.
-    Default: search "." match all -print all matches.
+    Default: search ".", match all, -print matches.
 
     -H  Follow command line symlinks         -L  Follow all symlinks
 
     Match filters:
-    -name  PATTERN  filename with wildcards   -iname      case insensitive -name
-    -path  PATTERN  path name with wildcards  -ipath      case insensitive -path
-    -user  UNAME    belongs to user UNAME     -nouser     user ID not known
-    -group GROUP    belongs to group GROUP    -nogroup    group ID not known
-    -perm  [-/]MODE permissions (-=min /=any) -prune      ignore contents of dir
-    -size  N[c]     512 byte blocks (c=bytes) -xdev       only this filesystem
-    -links N        hardlink count            -atime N[u] accessed N units ago
-    -ctime N[u]     created N units ago       -mtime N[u] modified N units ago
-    -newer FILE     newer mtime than FILE     -mindepth # at least # dirs down
-    -depth          ignore contents of dir    -maxdepth # at most # dirs down
-    -inum  N        inode number N            -empty      empty files and dirs
-    -type [bcdflps] (block, char, dir, file, symlink, pipe, socket)
+    -name  PATTERN   filename with wildcards   -iname      ignore case -name
+    -path  PATTERN   path name with wildcards  -ipath      ignore case -path
+    -user  UNAME     belongs to user UNAME     -nouser     user ID not known
+    -group GROUP     belongs to group GROUP    -nogroup    group ID not known
+    -perm  [-/]MODE  permissions (-=min /=any) -prune      ignore dir contents
+    -size  N[c]      512 byte blocks (c=bytes) -xdev       only this filesystem
+    -links N         hardlink count            -empty      empty files and dirs
+    -atime N[u]      accessed N units ago      -true       always true
+    -ctime N[u]      created N units ago       -false      always false
+    -mtime N[u]      modified N units ago      -executable access(X_OK) perm+ACL
+    -inum  N         inode number N            -readable   access(R_OK) perm+ACL
+    -context PATTERN security context          -depth      contents before dir
+    -samefile FILE   hardlink to FILE          -maxdepth N at most N dirs down
+    -newer    FILE   newer mtime than FILE     -mindepth N at least N dirs down
+    -newerXY  FILE   X=acm time > FILE's Y=acm time (Y=t: FILE is literal time)
+    -type [bcdflps]  type is (block, char, dir, file, symlink, pipe, socket)
 
-    Numbers N may be prefixed by a - (less than) or + (greater than). Units for
-    -Xtime are d (days, default), h (hours), m (minutes), or s (seconds).
+    Numbers N may be prefixed by - (less than) or + (greater than). Units for
+    -[acm]time are d (days, default), h (hours), m (minutes), or s (seconds).
 
     Combine matches with:
     !, -a, -o, ( )    not, and, or, group expressions
 
     Actions:
-    -print   Print match with newline  -print0    Print match with null
-    -exec    Run command with path     -execdir   Run command in file's dir
-    -ok      Ask before exec           -okdir     Ask before execdir
-    -delete  Remove matching file/dir
+    -print  Print match with newline  -print0        Print match with null
+    -exec   Run command with path     -execdir       Run command in file's dir
+    -ok     Ask before exec           -okdir         Ask before execdir
+    -delete Remove matching file/dir  -printf FORMAT Print using format string
+    -quit   Exit immediately
 
     Commands substitute "{}" with matched file. End with ";" to run each file,
     or "+" (next argument after "{}") to collect and run with multiple files.
+
+    -printf FORMAT characters are \ escapes and:
+    %b  512 byte blocks used
+    %f  basename            %g  textual gid          %G  numeric gid
+    %i  decimal inode       %l  target of symlink    %m  octal mode
+    %M  ls format type/mode %p  path to file         %P  path to file minus DIR
+    %s  size in bytes       %T@ mod time as unixtime
+    %u  username            %U  numeric uid          %Z  security context
 */
 
 #define FOR_find
@@ -60,6 +74,7 @@ GLOBALS(
   int topdir, xdev, depth;
   time_t now;
   long max_bytes;
+  char *start;
 )
 
 struct execdir_data {
@@ -189,7 +204,7 @@ static void execdir(struct dirtree *new, int flush)
       aa->execdir = bb;
     }
   }
-} 
+}
 
 // Call this with 0 for first pass argument parsing and syntax checking (which
 // populates argdata). Later commands traverse argdata (in order) when they
@@ -198,24 +213,32 @@ static int do_find(struct dirtree *new)
 {
   int pcount = 0, print = 0, not = 0, active = !!new, test = active, recurse;
   struct double_list *argdata = TT.argdata;
-  char *s, **ss;
+  char *s, **ss, *arg;
 
-  recurse = DIRTREE_COMEAGAIN|(DIRTREE_SYMFOLLOW*!!(toys.optflags&FLAG_L));
+  recurse = DIRTREE_STATLESS|DIRTREE_COMEAGAIN|DIRTREE_SYMFOLLOW*FLAG(L);
 
   // skip . and .. below topdir, handle -xdev and -depth
   if (new) {
+    // Handle stat failures first.
+    if (new->again&DIRTREE_STATLESS) {
+      if (!new->parent || errno != ENOENT) {
+        perror_msg("'%s'", s = dirtree_path(new, 0));
+        free(s);
+      }
+      return 0;
+    }
     if (new->parent) {
       if (!dirtree_notdotdot(new)) return 0;
       if (TT.xdev && new->st.st_dev != new->parent->st.st_dev) recurse = 0;
-    }
+    } else TT.start = new->name;
 
     if (S_ISDIR(new->st.st_mode)) {
       // Descending into new directory
-      if (!new->again) {
+      if (!(new->again&DIRTREE_COMEAGAIN)) {
         struct dirtree *n;
 
         for (n = new->parent; n; n = n->parent) {
-          if (n->st.st_ino==new->st.st_ino && n->st.st_dev==new->st.st_dev) {
+          if (same_file(&n->st, &new->st)) {
             error_msg("'%s': loop detected", s = dirtree_path(new, 0));
             free(s);
 
@@ -243,10 +266,10 @@ static int do_find(struct dirtree *new)
   // not: a pending ! applies to this test (only set if performing tests)
   // print: saw one of print/ok/exec, no need for default -print
 
-  if (TT.filter) for (ss = TT.filter; *ss; ss++) {
+  if (TT.filter) for (ss = TT.filter; (s = *ss); ss++) {
     int check = active && test;
 
-    s = *ss;
+    // if (!new) perform one-time setup, if (check) perform test
 
     // handle ! ( ) using toybuf as a stack
     if (*s != '-') {
@@ -284,7 +307,7 @@ static int do_find(struct dirtree *new)
       if (new && check)
         test = !unlinkat(dirtree_parentfd(new), new->name,
           S_ISDIR(new->st.st_mode) ? AT_REMOVEDIR : 0);
-    } else if (!strcmp(s, "depth")) TT.depth = 1;
+    } else if (!strcmp(s, "depth") || !strcmp(s, "d")) TT.depth = 1;
     else if (!strcmp(s, "o") || !strcmp(s, "or")) {
       if (not) goto error;
       if (active) {
@@ -294,6 +317,11 @@ static int do_find(struct dirtree *new)
     } else if (!strcmp(s, "not")) {
       if (check) not = !not;
       continue;
+    } else if (!strcmp(s, "true")) {
+      if (check) test = 1;
+    } else if (!strcmp(s, "false")) {
+      if (check) test = 0;
+
     // Mostly ignore NOP argument
     } else if (!strcmp(s, "a") || !strcmp(s, "and") || !strcmp(s, "noleaf")) {
       if (not) goto error;
@@ -319,22 +347,35 @@ static int do_find(struct dirtree *new)
         } else test = 0;
       }
     } else if (!strcmp(s, "nouser")) {
-      if (check) if (bufgetpwuid(new->st.st_uid)) test = 0;
+      if (check && bufgetpwuid(new->st.st_uid)) test = 0;
     } else if (!strcmp(s, "nogroup")) {
-      if (check) if (bufgetgrgid(new->st.st_gid)) test = 0;
+      if (check && bufgetgrgid(new->st.st_gid)) test = 0;
     } else if (!strcmp(s, "prune")) {
       if (check && S_ISDIR(new->st.st_mode) && !TT.depth) recurse = 0;
+    } else if (!strcmp(s, "executable") || !strcmp(s, "readable")) {
+      if (check && faccessat(dirtree_parentfd(new), new->name,
+          *s=='r' ? R_OK : X_OK, 0)) test = 0;
+    } else if (!strcmp(s, "quit")) {
+      if (check) {
+        execdir(0, 1);
+        xexit();
+      }
 
     // Remaining filters take an argument
     } else {
+      arg = *++ss;
       if (!strcmp(s, "name") || !strcmp(s, "iname")
-        || !strcmp(s, "path") || !strcmp(s, "ipath"))
+        || !strcmp(s, "wholename") || !strcmp(s, "iwholename")
+        || !strcmp(s, "path") || !strcmp(s, "ipath")
+        || !strcmp(s, "lname") || !strcmp(s, "ilname"))
       {
-        int i = (*s == 'i');
-        char *arg = ss[1], *path = 0, *name = new ? new->name : arg;
+        int i = (*s == 'i'), is_path = (s[i] != 'n');
+        char *path = 0, *name = new ? new->name : arg;
 
         // Handle path expansion and case flattening
-        if (new && s[i] == 'p') name = path = dirtree_path(new, 0);
+        if (new && s[i] == 'l')
+          name = path = xreadlinkat(dirtree_parentfd(new), new->name);
+        else if (new && is_path) name = path = dirtree_path(new, 0);
         if (i) {
           if ((check || !new) && name) name = strlower(name);
           if (!new) dlist_add(&TT.argdata, name);
@@ -342,16 +383,25 @@ static int do_find(struct dirtree *new)
         }
 
         if (check) {
-          test = !fnmatch(arg, name, FNM_PATHNAME*(s[i] == 'p'));
+          test = !fnmatch(arg, path ? name : basename(name),
+            FNM_PATHNAME*(!is_path));
           if (i) free(name);
         }
         free(path);
+      } else if (!CFG_TOYBOX_LSM_NONE && !strcmp(s, "context")) {
+        if (check) {
+          char *path = dirtree_path(new, 0), *context;
+
+          if (lsm_get_context(path, &context) != -1) {
+            test = !fnmatch(arg, context, 0);
+            free(context);
+          } else test = 0;
+          free(path);
+        }
       } else if (!strcmp(s, "perm")) {
         if (check) {
-          char *m = ss[1];
-          int match_min = *m == '-',
-              match_any = *m == '/';
-          mode_t m1 = string_to_mode(m+(match_min || match_any), 0),
+          int match_min = *arg == '-', match_any = *arg == '/';
+          mode_t m1 = string_to_mode(arg+(match_min || match_any), 0),
                  m2 = new->st.st_mode & 07777;
 
           if (match_min || match_any) m2 &= m1;
@@ -360,41 +410,44 @@ static int do_find(struct dirtree *new)
       } else if (!strcmp(s, "type")) {
         if (check) {
           int types[] = {S_IFBLK, S_IFCHR, S_IFDIR, S_IFLNK, S_IFIFO,
-                         S_IFREG, S_IFSOCK}, i = stridx("bcdlpfs", *ss[1]);
+                         S_IFREG, S_IFSOCK}, i;
 
-          if (i<0) error_exit("bad -type '%c'", *ss[1]);
-          if ((new->st.st_mode & S_IFMT) != types[i]) test = 0;
+          for (; *arg; arg++) {
+            if (*arg == ',') continue;
+            i = stridx("bcdlpfs", *arg);
+            if (i<0) error_exit("bad -type '%c'", *arg);
+            if ((new->st.st_mode & S_IFMT) == types[i]) break;
+          }
+          test = *arg;
         }
 
       } else if (strchr("acm", *s)
         && (!strcmp(s+1, "time") || !strcmp(s+1, "min")))
       {
         if (check) {
-          char *copy = ss[1];
           time_t thyme = (int []){new->st.st_atime, new->st.st_ctime,
                                   new->st.st_mtime}[stridx("acm", *s)];
-          int len = strlen(copy), uu, units = (s[1]=='m') ? 60 : 86400;
+          int len = strlen(arg), uu, units = (s[1]=='m') ? 60 : 86400;
 
-          if (len && -1!=(uu = stridx("dhms",tolower(copy[len-1])))) {
-            copy = xstrdup(copy);
-            copy[--len] = 0;
+          if (len && -1!=(uu = stridx("dhms",tolower(arg[len-1])))) {
+            arg = xstrdup(arg);
+            arg[--len] = 0;
             units = (int []){86400, 3600, 60, 1}[uu];
           }
-          test = compare_numsign(TT.now - thyme, units, copy);
-          if (copy != ss[1]) free(copy);
+          test = compare_numsign(TT.now - thyme, units, arg);
+          if (*ss != arg) free(arg);
         }
       } else if (!strcmp(s, "size")) {
-        if (check)
-          test = compare_numsign(new->st.st_size, 512, ss[1]);
+        if (check) test = compare_numsign(new->st.st_size, -512, arg) &&
+                          S_ISREG(new->st.st_mode);
       } else if (!strcmp(s, "links")) {
-        if (check) test = compare_numsign(new->st.st_nlink, 0, ss[1]);
+        if (check) test = compare_numsign(new->st.st_nlink, 0, arg);
       } else if (!strcmp(s, "inum")) {
-        if (check)
-          test = compare_numsign(new->st.st_ino, 0, ss[1]);
+        if (check) test = compare_numsign(new->st.st_ino, 0, arg);
       } else if (!strcmp(s, "mindepth") || !strcmp(s, "maxdepth")) {
         if (check) {
           struct dirtree *dt = new;
-          int i = 0, d = atolx(ss[1]);
+          int i = 0, d = atolx(arg);
 
           while ((dt = dt->parent)) i++;
           if (s[1] == 'i') {
@@ -406,40 +459,57 @@ static int do_find(struct dirtree *new)
           }
         }
       } else if (!strcmp(s, "user") || !strcmp(s, "group")
-              || !strcmp(s, "newer"))
+              || !strncmp(s, "newer", 5) || !strcmp(s, "samefile"))
       {
+        int macoff[] = {offsetof(struct stat, st_mtim),
+          offsetof(struct stat, st_atim), offsetof(struct stat, st_ctim)};
         struct {
           void *next, *prev;
           union {
             uid_t uid;
             gid_t gid;
             struct timespec tm;
-          } u;
+            struct dev_ino di;
+          };
         } *udl;
+        struct stat st;
 
         if (!new) {
-          if (ss[1]) {
+          if (arg) {
             udl = xmalloc(sizeof(*udl));
             dlist_add_nomalloc(&TT.argdata, (void *)udl);
 
-            if (*s == 'u') udl->u.uid = xgetuid(ss[1]);
-            else if (*s == 'g') udl->u.gid = xgetgid(ss[1]);
-            else {
-              struct stat st;
+            if (strchr("sn", *s)) {
+              if (*s=='n' && s[5] && (s[7] || !strchr("Bmac", s[5]) || !strchr("tBmac", s[6])))
+                goto error;
+              if (*s=='s' || !s[5] || s[6]!='t') {
+                xstat(arg, &st);
+                if (*s=='s') udl->di.dev = st.st_dev, udl->di.ino = st.st_ino;
+                else udl->tm = *(struct timespec *)(((char *)&st)
+                               + macoff[!s[5] ? 0 : stridx("ac", s[6])+1]);
+              } else if (s[6] == 't') {
+                unsigned nano;
 
-              xstat(ss[1], &st);
-              udl->u.tm = st.st_mtim;
-            }
+                xparsedate(arg, &(udl->tm.tv_sec), &nano, 1);
+                udl->tm.tv_nsec = nano;
+              }
+            } else if (*s == 'u') udl->uid = xgetuid(arg);
+            else udl->gid = xgetgid(arg);
           }
         } else {
           udl = (void *)llist_pop(&argdata);
           if (check) {
-            if (*s == 'u') test = new->st.st_uid == udl->u.uid;
-            else if (*s == 'g') test = new->st.st_gid == udl->u.gid;
+            if (*s == 'u') test = new->st.st_uid == udl->uid;
+            else if (*s == 'g') test = new->st.st_gid == udl->gid;
+            else if (*s == 's') test = same_dev_ino(&new->st, &udl->di);
             else {
-              test = new->st.st_mtim.tv_sec > udl->u.tm.tv_sec;
-              if (new->st.st_mtim.tv_sec == udl->u.tm.tv_sec)
-                test = new->st.st_mtim.tv_nsec > udl->u.tm.tv_nsec;
+              struct timespec *tm = (void *)(((char *)&new->st)
+                + macoff[!s[5] ? 0 : stridx("ac", s[5])+1]);
+
+              if (s[5] == 'B') test = 0;
+              else test = (tm->tv_sec == udl->tm.tv_sec)
+                ? tm->tv_nsec > udl->tm.tv_nsec
+                : tm->tv_sec > udl->tm.tv_sec;
             }
           }
         }
@@ -455,10 +525,10 @@ static int do_find(struct dirtree *new)
           int len;
 
           // catch "-exec" with no args and "-exec \;"
-          if (!ss[1] || !strcmp(ss[1], ";")) error_exit("'%s' needs 1 arg", s);
+          if (!arg || !strcmp(arg, ";")) error_exit("'%s' needs 1 arg", s);
 
           dlist_add_nomalloc(&TT.argdata, (void *)(aa = xzalloc(sizeof(*aa))));
-          aa->argstart = ++ss;
+          aa->argstart = ss;
           aa->curly = -1;
 
           // Record command line arguments to -exec
@@ -482,19 +552,19 @@ static int do_find(struct dirtree *new)
 
         // collect names and execute commands
         } else {
-          char *name, *ss1 = ss[1];
+          char *name;
           struct execdir_data *bb;
 
           // Grab command line exec argument list
           aa = (void *)llist_pop(&argdata);
-          ss += aa->arglen + 1;
+          ss += aa->arglen;
 
           if (!check) goto cont;
           // name is always a new malloc, so we can always free it.
           name = aa->dir ? xstrdup(new->name) : dirtree_path(new, 0);
 
           if (*s == 'o') {
-            fprintf(stderr, "[%s] %s", ss1, name);
+            fprintf(stderr, "[%s] %s", arg, name);
             if (!(test = yesno(0))) {
               free(name);
               goto cont;
@@ -520,17 +590,85 @@ static int do_find(struct dirtree *new)
               aa->plus = 1;
               toys.exitval |= flush_exec(new, aa);
             }
-          } else test = flush_exec(new, aa);
+          } else test = !flush_exec(new, aa);
         }
 
         // Argument consumed, skip the check.
         goto cont;
+      } else if (!strcmp(s, "printf")) {
+        char *fmt, *ff, next[32], buf[64], ch;
+        long ll;
+        int len;
+
+        print++;
+        if (check) for (fmt = arg; *fmt; fmt++) {
+          // Print the parts that aren't escapes
+          if (*fmt == '\\') {
+            unsigned u;
+
+            if (fmt[1] == 'c') break;
+            if ((u = unescape2(&fmt, 0))<128) putchar(u);
+            else printf("%.*s", (int)wcrtomb(buf, u, 0), buf);
+            fmt--;
+          } else if (*fmt != '%') putchar(*fmt);
+          else if (*++fmt == '%') putchar('%');
+          else {
+            fmt = next_printf(ff = fmt-1, 0);
+            if ((len = fmt-ff)>28) error_exit("bad %.*s", len+1, ff);
+            memcpy(next, ff, len);
+            ff = 0;
+            ch = *fmt;
+
+            // long long is its own stack size on LP64, so handle separately
+            if (ch == 'i' || ch == 's') {
+              strcpy(next+len, "lld");
+              printf(next, (ch == 'i') ? (long long)new->st.st_ino
+                : (long long)new->st.st_size);
+            } else {
+
+              // LP64 says these are all a single "long" argument to printf
+              strcpy(next+len, "s");
+              if (ch == 'G') next[len] = 'd', ll = new->st.st_gid;
+              else if (ch == 'm') next[len] = 'o', ll = new->st.st_mode&~S_IFMT;
+              else if (ch == 'U') next[len] = 'd', ll = new->st.st_uid;
+              else if (ch == 'f') ll = (long)new->name;
+              else if (ch == 'g') ll = (long)getgroupname(new->st.st_gid);
+              else if (ch == 'u') ll = (long)getusername(new->st.st_uid);
+              else if (ch == 'l') {
+                ll = (long)(ff = xreadlinkat(dirtree_parentfd(new), new->name));
+                if (!ll) ll = (long)"";
+              } else if (ch == 'M') {
+                mode_to_string(new->st.st_mode, buf);
+                ll = (long)buf;
+              } else if (ch == 'P') {
+                ch = *TT.start;
+                *TT.start = 0;
+                ll = (long)(ff = dirtree_path(new, 0));
+                *TT.start = ch;
+              } else if (ch == 'p') ll = (long)(ff = dirtree_path(new, 0));
+              else if (ch == 'T') {
+                if (*++fmt!='@') error_exit("bad -printf %%T: %%T%c", *fmt);
+                sprintf(buf, "%lld.%ld", (long long)new->st.st_mtim.tv_sec,
+                             new->st.st_mtim.tv_nsec);
+                ll = (long)buf;
+              } else if (ch == 'Z') {
+                char *path = dirtree_path(new, 0);
+
+                ll = (lsm_get_context(path, &ff) != -1) ? (long)ff : (long)"?";
+                free(path);
+              } else error_exit("bad -printf %%%c", ch);
+
+              printf(next, ll);
+              free(ff);
+            }
+          }
+        }
       } else goto error;
 
       // This test can go at the end because we do a syntax checking
       // pass first. Putting it here gets the error message (-unknown
       // vs -known noarg) right.
-      if (!*++ss) error_exit("'%s' needs 1 arg", --s);
+      if (!check && !arg) error_exit("'%s' needs 1 arg", s-1);
     }
 cont:
     // Apply pending "!" to result
@@ -543,33 +681,32 @@ cont:
     if (!print && test) do_print(new, '\n');
 
     if (S_ISDIR(new->st.st_mode)) execdir(new, 0);
- 
+
   } else dlist_terminate(TT.argdata);
 
   return recurse;
 
 error:
+  if (!*ss) --ss;
   error_exit("bad arg '%s'", *ss);
 }
 
 void find_main(void)
 {
   int i, len;
-  char **ss = toys.optargs;
+  char **ss = (char *[]){"."};
 
   TT.topdir = -1;
   TT.max_bytes = sysconf(_SC_ARG_MAX) - environ_bytes();
 
   // Distinguish paths from filters
   for (len = 0; toys.optargs[len]; len++)
-    if (strchr("-!(", *toys.optargs[len])) break;
+    if (*toys.optargs[len] && strchr("-!(", *toys.optargs[len])) break;
   TT.filter = toys.optargs+len;
 
   // use "." if no paths
-  if (!len) {
-    ss = (char *[]){"."};
-    len = 1;
-  }
+  if (len) ss = toys.optargs;
+  else len = 1;
 
   // first pass argument parsing, verify args match up, handle "evaluate once"
   TT.now = time(0);
@@ -577,7 +714,8 @@ void find_main(void)
 
   // Loop through paths
   for (i = 0; i < len; i++)
-    dirtree_flagread(ss[i], DIRTREE_SYMFOLLOW*!!(toys.optflags&(FLAG_H|FLAG_L)),
+    dirtree_flagread(ss[i],
+      DIRTREE_STATLESS|(DIRTREE_SYMFOLLOW*!!(toys.optflags&(FLAG_H|FLAG_L))),
       do_find);
 
   execdir(0, 1);
