@@ -4,28 +4,20 @@
  *
  * See http://opengroup.org/onlinepubs/9699919799/utilities/mkdir.html
 
-USE_MKDIR(NEWTOY(mkdir, "<1"USE_MKDIR_Z("Z:")"vp(parent)(parents)m:", TOYFLAG_BIN|TOYFLAG_UMASK))
+USE_MKDIR(NEWTOY(mkdir, "<1"SKIP_TOYBOX_LSM_NONE("Z:")"vp(parent)(parents)m:", TOYFLAG_BIN|TOYFLAG_UMASK|TOYFLAG_MOREHELP(!CFG_TOYBOX_LSM_NONE)))
 
 config MKDIR
   bool "mkdir"
   default y
   help
-    usage: mkdir [-vp] [-m MODE] [DIR...]
+    usage: mkdir [-vp] ![!-!Z! !c!o!n!t!e!x!t!]! [-m MODE] [DIR...]
 
     Create one or more directories.
 
     -m	Set permissions of directory to mode
     -p	Make parent directories as needed
     -v	Verbose
-
-config MKDIR_Z
-  bool
-  default y
-  depends on MKDIR && !TOYBOX_LSM_NONE
-  help
-    usage: [-Z context]
-
-    -Z	Set security context
+    !-Z	Set security context
 */
 
 #define FOR_mkdir
@@ -40,9 +32,7 @@ void mkdir_main(void)
   char **s;
   mode_t mode = (0777&~toys.old_umask);
 
-  if (CFG_MKDIR_Z && FLAG(Z))
-    if (0>lsm_set_create(TT.Z))
-      perror_exit("-Z '%s' failed", TT.Z);
+  if (FLAG(Z)) if (0>lsm_set_create(TT.Z)) perror_exit("-Z '%s' failed", TT.Z);
 
   if (TT.m) mode = string_to_mode(TT.m, 0777);
 
