@@ -4,24 +4,17 @@
  *
  * See http://opengroup.org/onlinepubs/9699919799/utilities/mkfifo.html
 
-USE_MKFIFO(NEWTOY(mkfifo, "<1"USE_MKFIFO_Z("Z:")"m:", TOYFLAG_USR|TOYFLAG_BIN))
+USE_MKFIFO(NEWTOY(mkfifo, "<1"SKIP_TOYBOX_LSM_NONE("Z:")"m:", TOYFLAG_USR|TOYFLAG_BIN|TOYFLAG_MOREHELP(!CFG_TOYBOX_LSM_NONE)))
 
 config MKFIFO
   bool "mkfifo"
   default y
   help
-    usage: mkfifo [NAME...]
+    usage: mkfifo ![!-!Z! !C!O!N!T!E!X!T!]! [NAME...]
 
     Create FIFOs (named pipes).
-
-config MKFIFO_Z
-  bool
-  default y
-  depends on MKFIFO && !TOYBOX_LSM_NONE
-  help
-    usage: mkfifo [-Z CONTEXT]
-
-    -Z	Security context
+    !
+    !-Z	Security context
 */
 
 #define FOR_mkfifo
@@ -40,9 +33,7 @@ void mkfifo_main(void)
   TT.mode = 0666;
   if (FLAG(m)) TT.mode = string_to_mode(TT.m, 0);
 
-  if (CFG_MKFIFO_Z && FLAG(Z))
-    if (0>lsm_set_create(TT.Z))
-      perror_exit("-Z '%s' failed", TT.Z);
+  if (FLAG(Z)) if (0>lsm_set_create(TT.Z)) perror_exit("-Z '%s' failed", TT.Z);
 
   for (s = toys.optargs; *s; s++)
     if (mknod(*s, S_IFIFO | TT.mode, 0) < 0) perror_msg_raw(*s);
