@@ -4,13 +4,13 @@
  *
  * See http://opengroup.org/onlinepubs/9699919799/utilities/nice.html
 
-USE_NICE(NEWTOY(nice, "^<1n#", TOYFLAG_USR|TOYFLAG_BIN))
+USE_NICE(NEWTOY(nice, "^<1n#", TOYFLAG_BIN))
 
 config NICE
   bool "nice"
   default y
   help
-    usage: nice [-n PRIORITY] command [args...]
+    usage: nice [-n PRIORITY] COMMAND...
 
     Run a command line at an increased or decreased scheduling priority.
 
@@ -18,6 +18,8 @@ config NICE
     priority) to 19 (lowest).  By default processes inherit their parent's
     niceness (usually 0).  By default this command adds 10 to the parent's
     priority.  Only root can set a negative niceness level.
+
+    -n	Add given adjustment to priority (default 10)
 */
 
 #define FOR_nice
@@ -32,7 +34,9 @@ void nice_main(void)
   if (!toys.optflags) TT.n = 10;
 
   errno = 0;
-  if (nice(TT.n)==-1 && errno) perror_exit("Can't set priority");
-
+  if (nice(TT.n)==-1 && errno) {
+    toys.exitval = 125;
+    perror_exit("Can't set priority");
+  }
   xexec(toys.optargs);
 }
