@@ -4,182 +4,139 @@
  *
  * See http://pubs.opengroup.org/onlinepubs/9699919799/utilities/sed.html
  *
+ * xform See https://www.gnu.org/software/tar/manual/html_section/transform.html
+ *
  * TODO: lines > 2G could wrap signed int length counters. Not just getline()
  * but N and s///
  * TODO: make y// handle unicode, unicode delimiters
  * TODO: handle error return from emit(), error_msg/exit consistently
  *       What's the right thing to do for -i when write fails? Skip to next?
  * test '//q' with no previous regex, also repeat previous regex?
+ *
+ * Deviations from POSIX: allow extended regular expressions with -r,
+ * editing in place with -i, separate with -s, NUL-delimited strings with -z,
+ * printf escapes in text, line continuations, semicolons after all commands,
+ * 2-address anywhere an address is allowed, "T" command, multiline
+ * continuations for [abc], \; to end [abc] argument before end of line.
+ * Explicit violations of stuff posix says NOT to do: N at EOF does default
+ * print, l escapes \n
+ * Added --tarxform mode to support tar --xform
 
-USE_SED(NEWTOY(sed, "(help)(version)e*f*inEr[+Er]", TOYFLAG_USR|TOYFLAG_BIN|TOYFLAG_LOCALE|TOYFLAG_NOHELP))
+USE_SED(NEWTOY(sed, "(help)(version)(tarxform)e*f*i:;nErz(null-data)s[+Er]", TOYFLAG_BIN|TOYFLAG_AUTOCONF))
 
 config SED
   bool "sed"
   default y
   help
-    usage: sed [-inrE] [-e SCRIPT]...|SCRIPT [-f SCRIPT_FILE]... [FILE...]
+    usage: sed [-inrszE] [-e SCRIPT]...|SCRIPT [-f SCRIPT_FILE]... [FILE...]
 
-    Stream editor. Apply one or more editing SCRIPTs to each line of input
-    (from FILE or stdin) producing output (by default to stdout).
+    Stream editor. Apply editing SCRIPTs to lines of input.
 
-    -e	add SCRIPT to list
-    -f	add contents of SCRIPT_FILE to list
-    -i	Edit each file in place
+    -e	Add SCRIPT to list
+    -f	Add contents of SCRIPT_FILE to list
+    -i	Edit each file in place (-iEXT keeps backup file with extension EXT)
     -n	No default output (use the p command to output matched lines)
     -r	Use extended regular expression syntax
-    -E	Alias for -r
+    -E	POSIX alias for -r
     -s	Treat input files separately (implied by -i)
+    -z	Use \0 rather than \n as input line separator
 
-    A SCRIPT is a series of one or more COMMANDs separated by newlines or
-    semicolons. All -e SCRIPTs are concatenated together as if separated
-    by newlines, followed by all lines from -f SCRIPT_FILEs, in order.
-    If no -e or -f SCRIPTs are specified, the first argument is the SCRIPT.
+    A SCRIPT is one or more COMMANDs separated by newlines or semicolons.
+    All -e SCRIPTs and -f SCRIPT_FILE contents are combined in order as if
+    separated by newlines. If no -e or -f then first argument is the SCRIPT.
 
-    Each COMMAND may be preceded by an address which limits the command to
-    apply only to the specified line(s). Commands without an address apply to
-    every line. Addresses are of the form:
+    COMMANDs apply to every line unless prefixed with an ADDRESS of the form:
 
-      [ADDRESS[,ADDRESS]]COMMAND
+      [ADDRESS[,ADDRESS]][!]COMMAND
 
-    The ADDRESS may be a decimal line number (starting at 1), a /regular
-    expression/ within a pair of forward slashes, or the character "$" which
-    matches the last line of input. (In -s or -i mode this matches the last
-    line of each file, otherwise just the last line of the last file.) A single
-    address matches one line, a pair of comma separated addresses match
-    everything from the first address to the second address (inclusive). If
-    both addresses are regular expressions, more than one range of lines in
-    each file can match.
+    ADDRESS is a line number (starting at 1), a /REGULAR EXPRESSION/, or $ for
+    last line (-s or -i makes it last line of each file). One address matches one
+    line, ADDRESS,ADDRESS matches from first to second inclusive. Two regexes can
+    match multiple ranges. ADDRESS,+N ends N lines later. ! inverts the match.
 
-    REGULAR EXPRESSIONS in sed are started and ended by the same character
-    (traditionally / but anything except a backslash or a newline works).
-    Backslashes may be used to escape the delimiter if it occurs in the
-    regex, and for the usual printf escapes (\abcefnrtv and octal, hex,
-    and unicode). An empty regex repeats the previous one. ADDRESS regexes
-    (above) require the first delimeter to be escaped with a backslash when
-    it isn't a forward slash (to distinguish it from the COMMANDs below).
+    REGULAR EXPRESSIONS start and end with the same character (anything but
+    backslash or newline). To use the delimiter in the regex escape it with a
+    backslash, and printf escapes (\abcefnrtv and octal, hex, and unicode) work.
+    An empty regex repeats the previous one. ADDRESS regexes require any
+    first delimiter except / to be \escaped to distinguish it from COMMANDs.
 
-    Sed mostly operates on individual lines one at a time. It reads each line,
-    processes it, and either writes it to the output or discards it before
-    reading the next line. Sed can remember one additional line in a separate
-    buffer (using the h, H, g, G, and x commands), and can read the next line
-    of input early (using the n and N command), but other than that command
-    scripts operate on individual lines of text.
+    Sed reads each line of input, processes it, and writes it out or discards it
+    before reading the next. Sed can remember one additional line in a separate
+    buffer (the h, H, g, G, and x commands), and can read the next line of input
+    early (the n and N commands), but otherwise operates on individual lines.
 
-    Each COMMAND starts with a single character. The following commands take
-    no arguments:
-
-      {  Start a new command block, continuing until a corresponding "}".
-         Command blocks may nest. If the block has an address, commands within
-         the block are only run for lines within the block's address range.
-
-      }  End command block (this command cannot have an address)
+    Each COMMAND starts with a single character. Commands with no arguments are:
 
+      !  Run this command when the ADDRESS _didn't_ match.
+      {  Start new command block, continuing until a corresponding "}".
+         Command blocks nest and can have ADDRESSes applying to the whole block.
+      }  End command block (this COMMAND cannot have an address)
       d  Delete this line and move on to the next one
          (ignores remaining COMMANDs)
-
       D  Delete one line of input and restart command SCRIPT (same as "d"
          unless you've glued lines together with "N" or similar)
-
       g  Get remembered line (overwriting current line)
-
       G  Get remembered line (appending to current line)
-
       h  Remember this line (overwriting remembered line)
-
       H  Remember this line (appending to remembered line, if any)
-
-      l  Print line, escaping \abfrtv (but not newline), octal escaping other
-         nonprintable characters, wrapping lines to terminal width with a
-         backslash, and appending $ to actual end of line.
-
-      n  Print default output and read next line, replacing current line
-         (If no next line available, quit processing script)
-
-      N  Append next line of input to this line, separated by a newline
-         (This advances the line counter for address matching and "=", if no
-         next line available quit processing script without default output)
-
+      l  Print line escaping \abfrtvn, octal escape other nonprintng chars,
+         wrap lines to terminal width with \, append $ to end of line.
+      n  Print default output and read next line over current line (quit at EOF)
+      N  Append \n and next line of input to this line. Quit at EOF without
+         default output. Advances line counter for ADDRESS and "=".
       p  Print this line
-
       P  Print this line up to first newline (from "N")
-
       q  Quit (print default output, no more commands processed or lines read)
-
       x  Exchange this line with remembered line (overwrite in both directions)
+      =  Print the current line number (plus newline)
+      #  Comment, ignores rest of this line of SCRIPT (until newline)
 
-      =  Print the current line number (followed by a newline)
-
-    The following commands (may) take an argument. The "text" arguments (to
-    the "a", "b", and "c" commands) may end with an unescaped "\" to append
-    the next line (for which leading whitespace is not skipped), and also
-    treat ";" as a literal character (use "\;" instead).
-
-      a [text]   Append text to output before attempting to read next line
-
-      b [label]  Branch, jumps to :label (or with no label, to end of SCRIPT)
-
-      c [text]   Delete line, output text at end of matching address range
-                 (ignores remaining COMMANDs)
-
-      i [text]   Print text
-
-      r [file]   Append contents of file to output before attempting to read
-                 next line.
-
-      s/S/R/F    Search for regex S, replace matched text with R using flags F.
-                 The first character after the "s" (anything but newline or
-                 backslash) is the delimiter, escape with \ to use normally.
-
-                 The replacement text may contain "&" to substitute the matched
-                 text (escape it with backslash for a literal &), or \1 through
-                 \9 to substitute a parenthetical subexpression in the regex.
-                 You can also use the normal backslash escapes such as \n and
-                 a backslash at the end of the line appends the next line.
-
-                 The flags are:
-
-                 [0-9]    A number, substitute only that occurrence of pattern
-                 g        Global, substitute all occurrences of pattern
-                 i        Ignore case when matching
-                 p        Print the line if match was found and replaced
-                 w [file] Write (append) line to file if match replaced
-
-      t [label]  Test, jump to :label only if an "s" command found a match in
-                 this line since last test (replacing with same text counts)
-
-      T [label]  Test false, jump only if "s" hasn't found a match.
-
-      w [file]   Write (append) line to file
+    Commands that take an argument:
 
+      : LABEL    Target for jump commands
+      a TEXT     Append text to output before reading next line
+      b LABEL    Branch, jumps to :LABEL (with no LABEL to end of SCRIPT)
+      c TEXT     Delete matching ADDRESS range and output TEXT instead
+      i TEXT     Insert text (output immediately)
+      r FILE     Append contents of FILE to output before reading next line.
+      s/S/R/F    Search for regex S replace match with R using flags F. Delimiter
+                 is anything but \n or \, escape with \ to use in S or R. Printf
+                 escapes work. Unescaped & in R becomes full matched text, \1
+                 through \9 = parenthetical subexpression from S. \ at end of
+                 line appends next line of SCRIPT. The flags in F are:
+                 [0-9]    A number N, substitute only Nth match
+                 g        Global, substitute all matches
+                 i/I      Ignore case when matching
+                 p        Print resulting line when match found and replaced
+                 w [file] Write (append) line to file when match replaced
+      t LABEL    Test, jump if s/// command matched this line since last test
+      T LABEL    Test false, jump to :LABEL only if no s/// found a match
+      w FILE     Write (append) line to file
       y/old/new/ Change each character in 'old' to corresponding character
                  in 'new' (with standard backslash escapes, delimiter can be
                  any repeated character except \ or \n)
 
-      : [label]  Labeled target for jump commands
-
-      #  Comment, ignore rest of this line of SCRIPT
-
-    Deviations from posix: allow extended regular expressions with -r,
-    editing in place with -i, separate with -s, printf escapes in text, line
-    continuations, semicolons after all commands, 2-address anywhere an
-    address is allowed, "T" command, multiline continuations for [abc],
-    \; to end [abc] argument before end of line.
+    The TEXT arguments (to a c i) may end with an unescaped "\" to append
+    the next line (leading whitespace is not skipped), and treat ";" as a
+    literal character (use "\;" instead).
 */
 
 #define FOR_sed
 #include "toys.h"
 
 GLOBALS(
+  char *i;
   struct arg_list *f, *e;
 
   // processed pattern list
   struct double_list *pattern;
 
-  char *nextline, *remember;
+  char *nextline, *remember, *tarxform;
   void *restart, *lastregex;
   long nextlen, rememberlen, count;
   int fdout, noeol;
-  unsigned xx;
+  unsigned xx, tarxlen, xflags;
+  char delim, xftype;
 )
 
 // Linked list of parsed sed commands. Offset fields indicate location where
@@ -195,23 +152,40 @@ struct sedcmd {
   int rmatch[2];  // offset of regex struct for prefix matches (/abc/,/def/p)
   int arg1, arg2, w; // offset of two arguments per command, plus s//w filename
   unsigned not, hit;
-  unsigned sflags; // s///flag bits: i=1, g=2, p=4
+  unsigned sflags; // s///flag bits, see SFLAG macros below
   char c; // action
 };
 
+#define SFLAG_i 1
+#define SFLAG_g 2
+#define SFLAG_p 4
+#define SFLAG_x 8
+#define SFLAG_slash 16
+#define SFLAG_R 32
+#define SFLAG_S 64
+#define SFLAG_H 128
+
 // Write out line with potential embedded NUL, handling eol/noeol
 static int emit(char *line, long len, int eol)
 {
-  int l, old = line[len];
+  int l = len, old = line[len];
 
-  if (TT.noeol && !writeall(TT.fdout, "\n", 1)) return 1;
+  if (FLAG(tarxform)) {
+    TT.tarxform = xrealloc(TT.tarxform, TT.tarxlen+len+TT.noeol+eol);
+    if (TT.noeol) TT.tarxform[TT.tarxlen++] = TT.delim;
+    memcpy(TT.tarxform+TT.tarxlen, line, len);
+    TT.tarxlen += len;
+    if (eol) TT.tarxform[TT.tarxlen++] = TT.delim;
+  } else {
+    if (TT.noeol && !writeall(TT.fdout, &TT.delim, 1)) return 1;
+    if (eol) line[len++] = TT.delim;
+    if (!len) return 0;
+    l = writeall(TT.fdout, line, len);
+    if (eol) line[len-1] = old;
+  }
   TT.noeol = !eol;
-  if (eol) line[len++] = '\n';
-  if (!len) return 0;
-  l = writeall(TT.fdout, line, len);
-  if (eol) line[len-1] = old;
   if (l != len) {
-    perror_msg("short write");
+    if (TT.fdout != 1) perror_msg("short write");
 
     return 1;
   }
@@ -228,7 +202,7 @@ static char *extend_string(char **old, char *new, int oldlen, int newlen)
 
   if (newline) newlen = -newlen;
   s = *old = xrealloc(*old, oldlen+newlen+newline+1);
-  if (newline) s[oldlen++] = '\n';
+  if (newline) s[oldlen++] = TT.delim;
   memcpy(s+oldlen, new, newlen);
   s[oldlen+newlen] = 0;
 
@@ -236,14 +210,14 @@ static char *extend_string(char **old, char *new, int oldlen, int newlen)
 }
 
 // An empty regex repeats the previous one
-static void *get_regex(void *trump, int offset)
+static void *get_regex(void *command, int offset)
 {
   if (!offset) {
     if (!TT.lastregex) error_exit("no previous regex");
     return TT.lastregex;
   }
 
-  return TT.lastregex = offset+(char *)trump;
+  return TT.lastregex = offset+(char *)command;
 }
 
 // Apply pattern to line from input file
@@ -254,32 +228,52 @@ static void sed_line(char **pline, long plen)
     int file;
     char *str;
   } *append = 0;
-  char *line = TT.nextline;
-  long len = TT.nextlen;
+  char *line;
+  long len;
   struct sedcmd *command;
   int eol = 0, tea = 0;
 
-  // Ignore EOF for all files before last unless -i
-  if (!pline && !(toys.optflags&FLAG_i)) return;
+  if (FLAG(tarxform)) {
+    if (!pline) return;
 
-  // Grab next line for deferred processing (EOF detection: we get a NULL
-  // pline at EOF to flush last line). Note that only end of _last_ input
-  // file matches $ (unless we're doing -i).
-  TT.nextline = 0;
-  TT.nextlen = 0;
-  if (pline) {
-    TT.nextline = *pline;
-    TT.nextlen = plen;
+    line = *pline;
+    len = plen;
     *pline = 0;
+    pline = 0;
+  } else {
+    line = TT.nextline;
+    len = TT.nextlen;
+
+    // Ignore EOF for all files before last unless -i or -s
+    if (!pline && !FLAG(i) && !FLAG(s)) return;
+
+    // Grab next line for deferred processing (EOF detection: we get a NULL
+    // pline at EOF to flush last line). Note that only end of _last_ input
+    // file matches $ (unless we're doing -i).
+    TT.nextline = 0;
+    TT.nextlen = 0;
+    if (pline) {
+      TT.nextline = *pline;
+      TT.nextlen = plen;
+      *pline = 0;
+    }
   }
 
   if (!line || !len) return;
-  if (line[len-1] == '\n') line[--len] = eol++;
+  if (line[len-1] == TT.delim) line[--len] = eol++;
+  if (FLAG(tarxform) && len) {
+    TT.xftype = line[--len];
+    line[len] = 0;
+  }
   TT.count++;
 
-  // The restart-1 is because we added one to make sure it wasn't NULL,
-  // otherwise N as last command would restart script
-  command = TT.restart ? ((struct sedcmd *)TT.restart)-1 : (void *)TT.pattern;
+  // To prevent N as last command from restarting script, we added 1 to restart
+  // so we'd use it here even when NULL. Alas, compilers that think C has
+  // references instead of pointers assume ptr-1 can never be NULL (demonstrably
+  // untrue) and inappropriately dead code eliminate, so use LP64 math until
+  // we get a -fpointers-are-not-references compiler option.
+  command = (void *)(TT.restart ? ((unsigned long)TT.restart)-1
+    : (unsigned long)TT.pattern);
   TT.restart = 0;
 
   while (command) {
@@ -301,20 +295,23 @@ static void sed_line(char **pline, long plen)
             if (line && !regexec0(rm, line, len, 0, 0, 0)) miss = 1;
           }
         } else if (lm > 0 && lm < TT.count) command->hit = 0;
+        else if (lm < -1 && TT.count == command->hit+(-lm-1)) command->hit = 0;
 
       // Start a new match?
       } else {
         if (!(lm = *command->lmatch)) {
           void *rm = get_regex(command, *command->rmatch);
 
-          if (line && !regexec0(rm, line, len, 0, 0, 0)) command->hit++;
-        } else if (lm == TT.count || (lm == -1 && !pline)) command->hit++;
+          if (line && !regexec0(rm, line, len, 0, 0, 0))
+            command->hit = TT.count;
+        } else if (lm == TT.count || (lm == -1 && !pline))
+          command->hit = TT.count;
 
         if (!command->lmatch[1] && !command->rmatch[1]) miss = 1;
-      } 
+      }
 
       // Didn't match?
-      lm = !(command->hit ^ command->not);
+      lm = !(command->not^!!command->hit);
 
       // Deferred disable from regex end match
       if (miss || command->lmatch[1] == TT.count) command->hit = 0;
@@ -373,7 +370,7 @@ static void sed_line(char **pline, long plen)
     } else if (c=='D') {
       // Delete up to \n or end of buffer
       str = line;
-      while ((str-line)<len) if (*(str++) == '\n') break;
+      while ((str-line)<len) if (*(str++) == TT.delim) break;
       len -= str - line;
       memmove(line, str, len);
 
@@ -389,11 +386,11 @@ static void sed_line(char **pline, long plen)
       continue;
     } else if (c=='g') {
       free(line);
-      line = xstrdup(TT.remember);
+      line = xmemdup(TT.remember, TT.rememberlen+1);
       len = TT.rememberlen;
     } else if (c=='G') {
       line = xrealloc(line, len+TT.rememberlen+2);
-      line[len++] = '\n';
+      line[len++] = TT.delim;
       memcpy(line+len, TT.remember, TT.rememberlen);
       line[len += TT.rememberlen] = 0;
     } else if (c=='h') {
@@ -402,7 +399,7 @@ static void sed_line(char **pline, long plen)
       TT.rememberlen = len;
     } else if (c=='H') {
       TT.remember = xrealloc(TT.remember, TT.rememberlen+len+2);
-      TT.remember[TT.rememberlen++] = '\n';
+      TT.remember[TT.rememberlen++] = TT.delim;
       memcpy(TT.remember+TT.rememberlen, line, len);
       TT.remember[TT.rememberlen += len] = 0;
     } else if (c=='i') {
@@ -424,24 +421,26 @@ static void sed_line(char **pline, long plen)
           emit(toybuf, off, 1);
           off = 0;
         }
-        x = stridx("\\\a\b\f\r\t\v", line[i]);
+        x = stridx("\\\a\b\f\r\t\v\n", line[i]);
         if (x != -1) {
           toybuf[off++] = '\\';
-          toybuf[off++] = "\\abfrtv"[x];
+          toybuf[off++] = "\\abfrtvn"[x];
         } else if (line[i] >= ' ') toybuf[off++] = line[i];
         else off += sprintf(toybuf+off, "\\%03o", line[i]);
       }
       toybuf[off++] = '$';
       emit(toybuf, off, 1);
     } else if (c=='n') {
-      TT.restart = command->next+1;
+      // The +1 forces restart processing even when next is null
+      TT.restart = (void *)(((unsigned long)command->next)+1);
 
       break;
     } else if (c=='N') {
       // Can't just grab next line because we could have multiple N and
       // we need to actually read ahead to get N;$p EOF detection right.
       if (pline) {
-        TT.restart = command->next+1;
+        // The +1 forces restart processing even when  next is null
+        TT.restart = (void *)(((unsigned long)command->next)+1);
         extend_string(&line, TT.nextline, len, -TT.nextlen);
         free(TT.nextline);
         TT.nextline = line;
@@ -450,42 +449,57 @@ static void sed_line(char **pline, long plen)
       }
 
       // Pending append goes out right after N
-      goto done; 
+      goto done;
     } else if (c=='p' || c=='P') {
-      char *l = (c=='P') ? strchr(line, '\n') : 0;
+      char *l = (c=='P') ? strchr(line, TT.delim) : 0;
 
       if (emit(line, l ? l-line : len, eol)) break;
-    } else if (c=='q') {
+    } else if (c=='q' || c=='Q') {
       if (pline) *pline = (void *)1;
       free(TT.nextline);
+      if (!toys.exitval && command->arg1)
+        toys.exitval = atoi(command->arg1+(char *)command);
       TT.nextline = 0;
       TT.nextlen = 0;
+      if (c=='Q') line = 0;
 
       break;
     } else if (c=='s') {
-      char *rline = line, *new = command->arg2 + (char *)command, *swap, *rswap;
+      char *rline = line, *new = command->arg2 + (char *)command, *l2 = 0;
       regmatch_t *match = (void *)toybuf;
       regex_t *reg = get_regex(command, command->arg1);
-      int mflags = 0, count = 0, zmatch = 1, rlen = len, mlen, off, newlen;
+      int mflags = 0, count = 0, l2used = 0, zmatch = 1, l2l = len, l2old = 0,
+        bonk = 0, mlen, off, newlen;
+
+      // Skip suppressed --tarxform types
+      if (TT.xftype && (command->sflags & (SFLAG_R<<stridx("rsh", TT.xftype))));
+
+      // Loop finding match in remaining line (up to remaining len)
+      else while (!regexec0(reg, rline, len-(rline-line), 10, match, mflags)) {
+        mlen = match[0].rm_eo-match[0].rm_so;
+
+        // xform matches ending in / aren't allowed to match entire line
+        if ((command->sflags & SFLAG_slash) && mlen==len) {
+          while (len && ++bonk && line[--len]=='/');
+          continue;
+        }
 
-      // Find match in remaining line (up to remaining len)
-      while (!regexec0(reg, rline, rlen, 10, match, mflags)) {
         mflags = REG_NOTBOL;
 
         // Zero length matches don't count immediately after a previous match
-        mlen = match[0].rm_eo-match[0].rm_so;
         if (!mlen && !zmatch) {
-          if (!rlen--) break;
-          rline++;
+          if (rline-line == len) break;
+          if (l2) l2[l2used++] = *rline++;
           zmatch++;
           continue;
         } else zmatch = 0;
 
         // If we're replacing only a specific match, skip if this isn't it
-        off = command->sflags>>3;
+        off = command->sflags>>8;
         if (off && off != ++count) {
+          if (l2) memcpy(l2+l2used, rline, match[0].rm_eo);
+          l2used += match[0].rm_eo;
           rline += match[0].rm_eo;
-          rlen -= match[0].rm_eo;
 
           continue;
         }
@@ -506,13 +520,19 @@ static void sed_line(char **pline, long plen)
           newlen += match[cc].rm_eo-match[cc].rm_so;
         }
 
-        // Allocate new size, copy start/end around match. (Can't extend in
-        // place because backrefs may refer to text after it's overwritten.)
-        len += newlen-mlen;
-        swap = xmalloc(len+1);
-        rswap = swap+(rline-line)+match[0].rm_so;
-        memcpy(swap, line, (rline-line)+match[0].rm_so);
-        memcpy(rswap+newlen, rline+match[0].rm_eo, (rlen -= match[0].rm_eo)+1);
+        // Copy changed data to new string
+
+        // Adjust allocation size of new string, copy data we know we'll keep
+        l2l += newlen-mlen;
+        if ((mlen = l2l|0xfff) > l2old) {
+          l2 = xrealloc(l2, ++mlen);
+          if (l2used && !l2old) memcpy(l2, rline-l2used, l2used);
+          l2old = mlen;
+        }
+        if (match[0].rm_so) {
+          memcpy(l2+l2used, rline, match[0].rm_so);
+          l2used += match[0].rm_so;
+        }
 
         // copy in new replacement text
         for (off = mlen = 0; new[off]; off++) {
@@ -521,33 +541,42 @@ static void sed_line(char **pline, long plen)
           if (new[off] == '\\') {
             cc = new[++off] - '0';
             if (cc<0 || cc>9) {
-              if (!(rswap[mlen++] = unescape(new[off])))
-                rswap[mlen-1] = new[off];
+              if (!(l2[l2used+mlen++] = unescape(new[off])))
+                l2[l2used+mlen-1] = new[off];
 
               continue;
-            } else if (match[cc].rm_so == -1) error_exit("no s//\\%d/", cc);
+            } else if (cc > reg->re_nsub) error_exit("no s//\\%d/", cc);
           } else if (new[off] != '&') {
-            rswap[mlen++] = new[off];
+            l2[l2used+mlen++] = new[off];
 
             continue;
           }
 
-          ll = match[cc].rm_eo-match[cc].rm_so;
-          memcpy(rswap+mlen, rline+match[cc].rm_so, ll);
-          mlen += ll;
+          if (match[cc].rm_so != -1) {
+            ll = match[cc].rm_eo-match[cc].rm_so;
+            memcpy(l2+l2used+mlen, rline+match[cc].rm_so, ll);
+            mlen += ll;
+          }
         }
+        l2used += newlen;
+        rline += match[0].rm_eo;
 
-        rline = rswap+newlen;
+        if (!(command->sflags & SFLAG_g)) break;
+      }
+      len += bonk;
+
+      // If we made any changes, finish off l2 and swap it for line
+      if (l2) {
+        // grab trailing unmatched data and null terminator, swap with original
+        mlen = len-(rline-line);
+        memcpy(l2+l2used, rline, mlen+1);
+        len = l2used + mlen;
         free(line);
-        line = swap;
-
-        // Stop after first substitution unless we have flag g
-        if (!(command->sflags & 2)) break;
+        line = l2;
       }
 
       if (mflags) {
-        // flag p
-        if (command->sflags & 4) emit(line, len, eol);
+        if (command->sflags & SFLAG_p) emit(line, len, eol);
 
         tea = 1;
         if (command->w) goto writenow;
@@ -557,6 +586,8 @@ static void sed_line(char **pline, long plen)
       char *name;
 
 writenow:
+      if (FLAG(tarxform)) error_exit("tilt");
+
       // Swap out emit() context
       fd = TT.fdout;
       noeol = TT.noeol;
@@ -594,15 +625,16 @@ writenow:
       }
     } else if (c=='=') {
       sprintf(toybuf, "%ld", TT.count);
-      emit(toybuf, strlen(toybuf), 1);
+      if (emit(toybuf, strlen(toybuf), 1)) break;
     }
 
     command = command->next;
   }
 
-  if (line && !(toys.optflags & FLAG_n)) emit(line, len, eol);
-
 done:
+  if (line && !FLAG(n)) emit(line, len, eol);
+
+  // TODO: should "sed -z ax" use \n instead of NUL?
   if (dlist_terminate(append)) while (append) {
     struct append *a = append->next;
 
@@ -611,7 +643,7 @@ done:
 
       // Force newline if noeol pending
       if (fd != -1) {
-        if (TT.noeol) xwrite(TT.fdout, "\n", 1);
+        if (TT.noeol) xwrite(TT.fdout, &TT.delim, 1);
         TT.noeol = 0;
         xsendfile(fd, TT.fdout);
         close(fd);
@@ -622,27 +654,40 @@ done:
     append = a;
   }
   free(line);
+
+  if (TT.tarxlen) {
+    dprintf(TT.fdout, "%08x", --TT.tarxlen);
+    writeall(TT.fdout, TT.tarxform, TT.tarxlen);
+    TT.tarxlen = 0;
+  }
 }
 
 // Callback called on each input file
 static void do_sed_file(int fd, char *name)
 {
-  int i = toys.optflags & FLAG_i;
-  char *tmp;
-
-  if (i) {
-    struct sedcmd *command;
+  char *tmp, *s;
 
+  if (FLAG(i)) {
     if (!fd) return error_msg("-i on stdin");
     TT.fdout = copy_tempfile(fd, name, &tmp);
+  }
+  if (FLAG(i) || FLAG(s)) {
+    struct sedcmd *command;
+
     TT.count = 0;
     for (command = (void *)TT.pattern; command; command = command->next)
       command->hit = 0;
   }
-  do_lines(fd, sed_line);
-  if (i) {
+  do_lines(fd, TT.delim, sed_line);
+  if (FLAG(i)) {
+    if (TT.i && *TT.i) {
+      xrename(name, s = xmprintf("%s%s", name, TT.i));
+      free(s);
+    }
     replace_tempfile(-1, TT.fdout, &tmp);
     TT.fdout = 1;
+  }
+  if (FLAG(i) || FLAG(s)) {
     TT.nextline = 0;
     TT.nextlen = TT.noeol = 0;
   }
@@ -757,8 +802,17 @@ static void parse_pattern(char **pline, long len)
     }
     if (!*line) return;
 
-    // We start by writing data into toybuf. Later we'll allocate the
-    // ex
+    if (FLAG(tarxform) && strstart(&line, "flags=")) {
+      TT.xflags = 7;
+      while (0<=(i = stridx("rRsShH", *line))) {
+        if (i&1) TT.xflags |= 1<<(i>>1);
+        else TT.xflags &= ~(1<<(i>>1));
+        line++;
+      }
+      continue;
+    }
+
+    // Start by writing data into toybuf.
 
     errstart = line;
     memset(toybuf, 0, sizeof(struct sedcmd));
@@ -770,7 +824,10 @@ static void parse_pattern(char **pline, long len)
       if (*line == ',') line++;
       else if (i) break;
 
-      if (isdigit(*line)) command->lmatch[i] = strtol(line, &line, 0);
+      if (i && *line == '+' && isdigit(line[1])) {
+        line++;
+        command->lmatch[i] = -2-strtol(line, &line, 0);
+      } else if (isdigit(*line)) command->lmatch[i] = strtol(line, &line, 0);
       else if (*line == '$') {
         command->lmatch[i] = -1;
         line++;
@@ -780,7 +837,7 @@ static void parse_pattern(char **pline, long len)
         if (!(s = unescape_delimited_string(&line, 0))) goto error;
         if (!*s) command->rmatch[i] = 0;
         else {
-          xregcomp((void *)reg, s, (toys.optflags & FLAG_r)*REG_EXTENDED);
+          xregcomp((void *)reg, s, REG_EXTENDED*FLAG(r));
           command->rmatch[i] = reg-toybuf;
           reg += sizeof(regex_t);
         }
@@ -791,17 +848,18 @@ static void parse_pattern(char **pline, long len)
     while (isspace(*line)) line++;
     if (!*line) break;
 
-    while (*line == '!') {
+    if (*line == '!') {
       command->not = 1;
       line++;
     }
     while (isspace(*line)) line++;
+    if (!*line) break;
 
     c = command->c = *(line++);
     if (strchr("}:", c) && i) break;
-    if (strchr("aiqr=", c) && i>1) break;
+    if (strchr("aiqQr=", c) && i>1) break;
 
-    // Add step to pattern
+    // Allocate memory and copy out of toybuf now that we know how big it is
     command = xmemdup(toybuf, reg-toybuf);
     reg = (reg-toybuf) + (char *)command;
 
@@ -811,6 +869,7 @@ static void parse_pattern(char **pline, long len)
       if (!TT.nextlen--) break;
     } else if (c == 's') {
       char *end, delim = 0;
+      int flags;
 
       // s/pattern/replacement/flags
 
@@ -856,25 +915,36 @@ resume_s:
       i = command->arg1;
       command->arg1 = command->arg2;
       command->arg2 = i;
+      command->sflags = TT.xflags*SFLAG_R;
 
       // get flags
       for (line++; *line; line++) {
         long l;
 
         if (isspace(*line) && *line != '\n') continue;
-
-        if (0 <= (l = stridx("igp", *line))) command->sflags |= 1<<l;
-        else if (!(command->sflags>>3) && 0<(l = strtol(line, &line, 10))) {
-          command->sflags |= l << 3;
+        if (0 <= (l = stridx("igpx", *line))) command->sflags |= 1<<l;
+        else if (*line == 'I') command->sflags |= 1<<0;
+        else if (FLAG(tarxform) && 0 <= (l = stridx("RSH", *line)))
+          command->sflags |= SFLAG_R<<l;
+        // Given that the default is rsh all enabled... why do these exist?
+        else if (FLAG(tarxform) && 0 <= (l = stridx("rsh", *line)))
+          command->sflags &= ~(SFLAG_R<<l);
+        else if (!(command->sflags>>8) && 0<(l = strtol(line, &line, 10))) {
+          command->sflags |= l << 8;
           line--;
         } else break;
       }
+      flags = (FLAG(r) || (command->sflags & SFLAG_x)) ? REG_EXTENDED : 0;
+      if (command->sflags & SFLAG_i) flags |= REG_ICASE;
 
       // We deferred actually parsing the regex until we had the s///i flag
       // allocating the space was done by extend_string() above
       if (!*TT.remember) command->arg1 = 0;
-      else xregcomp((void *)(command->arg1 + (char *)command), TT.remember,
-        ((toys.optflags & FLAG_r)*REG_EXTENDED)|((command->sflags&1)*REG_ICASE));
+      else {
+        xregcomp((void *)(command->arg1+(char *)command), TT.remember, flags);
+        if (FLAG(tarxform) && TT.remember[strlen(TT.remember)-1]=='/')
+          command->sflags |= SFLAG_slash;
+      }
       free(TT.remember);
       TT.remember = 0;
       if (*line == 'w') {
@@ -896,7 +966,7 @@ writenow:
       for (cc = line; *cc; cc++) if (*cc == '\\' && cc[1] == ';') break;
       delim = *cc;
       *cc = 0;
-      fd = xcreate(line, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+      fd = xcreate(line, O_WRONLY|O_CREAT|O_TRUNC|O_APPEND, 0644);
       *cc = delim;
 
       command->w = reg - (char *)command;
@@ -926,7 +996,7 @@ writenow:
       if (len != strlen(s)) goto error;
       reg = extend_string((void *)&command, s, reg-(char*)command, len);
       free(s);
-    } else if (strchr("abcirtTw:", c)) {
+    } else if (strchr("abcirtTqQw:", c)) {
       int end;
 
       // trim leading spaces
@@ -937,12 +1007,20 @@ writenow:
 resume_a:
       command->hit = 0;
 
-      // btT: end with space or semicolon, aicrw continue to newline.
-      if (!(end = strcspn(line, strchr(":btT", c) ? "; \t\r\n\v\f" : "\n"))) {
-        // Argument's optional for btT
-        if (strchr("btT", c)) continue;
+      // btTqQ: end with space or semicolon, aicrw continue to newline.
+      if (!(end = strcspn(line, strchr(":btTqQ", c) ? "}; \t\r\n\v\f" : "\n"))){
+        // Argument's optional for btTqQ
+        if (strchr("btTqQ", c)) continue;
         else if (!command->arg1) break;
       }
+      // Error checking: qQ can only have digits after them
+      if (c=='q' || c=='Q') {
+        for (i = 0; i<end && isdigit(line[i]); i++);
+        if (i != end) {
+          line += i;
+          break;
+        }
+      }
 
       // Extend allocation to include new string. We use offsets instead of
       // pointers so realloc() moving stuff doesn't break things. Ok to write
@@ -980,28 +1058,25 @@ resume_a:
       } else line += end;
 
     // Commands that take no arguments
-    } else if (!strchr("{dDgGhHlnNpPqx=", c)) break;
+    } else if (!strchr("{dDgGhHlnNpPx=", c)) break;
   }
 
 error:
   error_exit("bad pattern '%s'@%ld (%c)", errstart, line-errstart+1L, *line);
 }
 
+// Is the pointer "find" within the string "range".
+static int instr(char *find, char *range)
+{
+  return find>=range && range+strlen(range)>=find;
+}
+
 void sed_main(void)
 {
-  struct arg_list *al;
-  char **args = toys.optargs;
+  char **args = toys.optargs, **aa;
 
-  // Lie to autoconf when it asks stupid questions, so configure regexes
-  // that look for "GNU sed version %f" greater than some old buggy number
-  // don't fail us for not matching their narrow expectations.
-  if (toys.optflags & FLAG_version) {
-    xprintf("This is not GNU sed version 9.0\n");
-    return;
-  }
-
-  // Handling our own --version means we handle our own --help too.
-  if (toys.optflags&FLAG_help) help_exit(0);
+  if (FLAG(tarxform)) toys.optflags |= FLAG_z;
+  if (!FLAG(z)) TT.delim = '\n';
 
   // Parse pattern into commands.
 
@@ -1011,14 +1086,20 @@ void sed_main(void)
     (TT.e = xzalloc(sizeof(struct arg_list)))->arg = *(args++);
   }
 
-  // Option parsing infrastructure can't interlace "-e blah -f blah -e blah"
-  // so handle all -e, then all -f. (At least the behavior's consistent.)
-
-  for (al = TT.e; al; al = al->next) parse_pattern(&al->arg, strlen(al->arg));
+  // -e and -f care about order, so use argv[] to recreate original order
+  for (aa = toys.argv+1; *aa; aa++) {
+    if (TT.e && instr(TT.e->arg, *aa)) {
+      parse_pattern(&TT.e->arg, strlen(TT.e->arg));
+      free(llist_pop(&TT.e));
+    }
+    if (TT.f && instr(TT.f->arg, *aa)) {
+      do_lines(xopenro(TT.f->arg), TT.delim, parse_pattern);
+      free(llist_pop(&TT.f));
+    }
+  }
   parse_pattern(0, 0);
-  for (al = TT.f; al; al = al->next) do_lines(xopenro(al->arg), parse_pattern);
   dlist_terminate(TT.pattern);
-  if (TT.nextlen) error_exit("no }");  
+  if (TT.nextlen) error_exit("no }");
 
   TT.fdout = 1;
   TT.remember = xstrdup("");
@@ -1027,10 +1108,10 @@ void sed_main(void)
   loopfiles_rw(args, O_RDONLY|WARN_ONLY, 0, do_sed_file);
 
   // Provide EOF flush at end of cumulative input for non-i mode.
-  if (!(toys.optflags & FLAG_i)) {
-    toys.optflags |= FLAG_i;
+  if (!FLAG(i) && !FLAG(s)) {
+    toys.optflags |= FLAG_s;
     sed_line(0, 0);
   }
 
-  // todo: need to close fd when done for TOYBOX_FREE?
+  // TODO: need to close fd when done for TOYBOX_FREE?
 }
