@@ -403,10 +403,10 @@ static int add_to_tar(struct dirtree *node)
       if (len>999999 || (sz && len>sz)) len = -1, errno = E2BIG;
       if (buf || len<1) {
         if (len>0) {
-          strcpy(buf+start+sz, "\n");
-          write_prefix_block(buf, start+sz+2, 'x');
+          strcpy(buf+start+sz-1, "\n");
+          write_prefix_block(buf, start+sz, 'x');
         } else if (errno==ENODATA || errno==ENOTSUP) len = 0;
-        if (len) perror_msg("getfilecon %s", name);
+        if (len<0) perror_msg("getfilecon %s", name);
 
         free(buf);
         break;
@@ -415,7 +415,7 @@ static int add_to_tar(struct dirtree *node)
       // Allocate buffer. Length includes prefix: calculate twice (wrap 99->100)
       temp = snprintf(0, 0, "%d", sz = (start = 22)+len+1);
       start += temp + (temp != snprintf(0, 0, "%d", temp+sz));
-      buf = xmprintf("%u RHT.%s=%.*s", start+len+1, sec, sz = len, "");
+      buf = xmprintf("%u RHT.%s=%.*s", start+len, sec, sz = len, "");
     }
   }
 
@@ -1045,7 +1045,7 @@ void tar_main(void)
 
   // If include file list empty, don't create empty archive
   if (FLAG(c)) {
-    if (!TT.incl) error_exit("empty archive");
+    if (!TT.incl && !FLAG(T)) error_exit("empty archive");
     TT.fd = 1;
   }
 
@@ -1100,7 +1100,7 @@ void tar_main(void)
         // detect gzip and bzip signatures
         if (SWAP_BE16(*(short *)hdr)==0x1f8b) toys.optflags |= FLAG_z;
         else if (!smemcmp(hdr, "BZh", 3)) toys.optflags |= FLAG_j;
-	else if (!smemcmp(hdr, "\x28\xb5\x2f\xfd", 4)) toys.optflags|=FLAG_Z;
+        else if (!smemcmp(hdr, "\x28\xb5\x2f\xfd", 4)) toys.optflags|=FLAG_Z;
         else if (peek_be(hdr, 7) == 0xfd377a585a0000ULL) toys.optflags |= FLAG_J;
         else error_exit("Not tar");
 
@@ -1194,7 +1194,7 @@ void tar_main(void)
       close(TT.fd);
       TT.fd = pipefd[0];
     }
-    do {
+    if (dl) do {
       TT.warn = 1;
       dirtree_flagread(dl->data,
         DIRTREE_SYMFOLLOW*FLAG(h)|DIRTREE_BREADTH*(FLAG(sort)|FLAG(s)),
